/*
    lm_use.c
    Copyright 1998, 1999 by Richard Dawe

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    ---

    This are Lan Manager DOS API functions for using shares. To use these
    functions requires the usage of some DPMI functions and DOS buffers. The
    information returned in buffers by lm_NetUseEnum() and lm_NetUseGetInfo()
    is in the form of pointers in the DOS area. This means one has to use
    movedata() with _dos_ds to extract the information.
*/

#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <dpmi.h>
#include <sys/movedata.h>
#include <sys/segments.h>
#include <sys/farptr.h>

#include "lm_use.h"

/* DPMI buffer globals */
int lm_bufsel = 0;
int lm_bufseg = 0;

/* Initialisation status */
int lm_inited = 0;

void lm_uninit_wrapper (void);

/* -----------
   - lm_init -
   ----------- */

int lm_init (void)
{
    if (lm_inited) return(lm_inited);

    /* Not init'd, so set up safe values. */
    lm_bufsel = lm_bufseg = 0;

    if (!lm_inited) {
        /* Allocate DOS memory */
        lm_bufseg = __dpmi_allocate_dos_memory(LM_BUFPARAS, &lm_bufsel);
        if (lm_bufseg == -1) return(0);

        /* Setup atexit() function */
        atexit(lm_uninit_wrapper);

        /* Done */
        lm_inited = 1;
    }

    return(lm_inited);
}

/* -------------
   - lm_uninit -
   ------------- */

int lm_uninit (void)
{
    if (lm_bufsel != 0) __dpmi_free_dos_memory(lm_bufsel);

    /* Done now */
    lm_inited = 0;
    return(1);
}

void lm_uninit_wrapper (void) { lm_uninit(); }

/* -----------------
   - lm_NetUseEnum -
   ----------------- */

int lm_NetUseEnum (int level, void *buf, int buflen)
{
    __dpmi_regs r;
    int count, itemsize;

    lm_init();

    /* Check params */
    if ( (level != 0) && (level != 1) ) return(-1);
    if (buf == NULL) return(-1);
    if (buflen < 14) return(-1);
    if (buflen > LM_BUFSIZE) return(-1);
   
    /* Make the call */
    bzero(&r, sizeof(r));
    r.x.ax = 0x5F46;
    r.x.bx = level;
    r.x.cx = buflen;
    r.x.es = lm_bufseg;
    r.x.di = 0;

    __dpmi_int(0x21, &r);

    if ((r.x.ax & 1) != 0) return(-1);
    count = r.x.cx;

    itemsize = (level == 0) ? sizeof(LM_USE_INFO_0) : sizeof(LM_USE_INFO_1);
    if (count > 0) movedata(lm_bufsel, 0, _my_ds(), (int) buf, count*itemsize);

    return(count);
}

/* ----------------
   - lm_NetUseAdd -
   ---------------- */

int lm_NetUseAdd (char *devname, char *remotename, char *passwd, int type)
{
    LM_USE_INFO_1 ui;
    __dpmi_regs r;
    char mynull[] = "";

    /* TODO: Parameter checking */
    if (passwd == NULL) passwd = mynull;

    /* Set up use info structure */
    bzero(&ui, sizeof(LM_USE_INFO_1));

    memcpy(&ui.devname, devname, strlen(devname));

    ui.remotename_seg = lm_bufseg;
    ui.remotename_off = sizeof(LM_USE_INFO_1);

    /* No password => null password password - still needs memory + pointer */
    ui.passwd_seg = lm_bufseg;
    ui.passwd_off = sizeof(LM_USE_INFO_1) + strlen(remotename) + 1;

    /* Don't fill out link status - irrelevant */
    ui.type = type;

    /* Stuff all data into DOS buffer */
    movedata(_my_ds(), (int) &ui,        lm_bufsel, 0,                     sizeof(LM_USE_INFO_1));
    movedata(_my_ds(), (int) remotename, lm_bufsel, sizeof(LM_USE_INFO_1), strlen(remotename) + 1);

    movedata(_my_ds(), (int) passwd, lm_bufsel,
             sizeof(LM_USE_INFO_1) + strlen(remotename) + 1, strlen(passwd) + 1);

    /* Make call */
    bzero(&r, sizeof(r));
    r.x.ax = 0x5F47;
    r.x.bx = 1;                     /* No level choice! */
    r.x.cx = sizeof(LM_USE_INFO_1);
    r.x.es = lm_bufseg;
    r.x.di = 0;

    __dpmi_int(0x21, &r);

    /* TODO: Decent errors */
    if ((r.x.flags & 1) != 0) return(-1); else return(0);
}

/* ----------------
   - lm_NetUseDel -
   ---------------- */

int lm_NetUseDel (char *name, int force)
{
    __dpmi_regs r;

    /* Copy device name */
    movedata(_my_ds(), (int) name, lm_bufsel, 0, strlen(name) + 1);

    /* Make call */
    bzero(&r, sizeof(r));
    r.x.ax = 0x5F48;
    r.x.bx = force;
    r.x.es = lm_bufseg;
    r.x.di = 0;

    __dpmi_int(0x21, &r);

    if ((r.x.flags & 1) == 1) return(-1); else return(0);
}

/* --------------------
   - lm_NetUseGetInfo -
   -------------------- */

int lm_NetUseGetInfo (char *devname, int level, void *buf, int buflen)
{
    __dpmi_regs r;
    int destbuflen;

    /* Check parameters */
    if ( (buf == NULL) || (devname == NULL) ) {
        errno = EINVAL;
        return(-1);
    }

    if ( (level != 0) && (level != 1) ) {
        errno = EINVAL;
        return(-1);
    }

    if (   ( (level == 0) && (buflen < sizeof(LM_USE_INFO_0)) )
        || ( (level == 1) && (buflen < sizeof(LM_USE_INFO_1)) ) ) {
        errno = ENOBUFS;
        return(-1);
    }

    /* Set up parameter block */
    destbuflen = (level == 0) ? sizeof(LM_USE_INFO_0) : sizeof(LM_USE_INFO_1);

    _farpokew(lm_bufsel,   0, 0xC);
    _farpokew(lm_bufsel,   2, lm_bufseg);
    _farpokew(lm_bufsel,   4, level);
    _farpokew(lm_bufsel,   6, 0xC + strlen(devname) + 1);
    _farpokew(lm_bufsel,   8, lm_bufseg);
    _farpokew(lm_bufsel, 0xA, destbuflen);

    /* Copy the device name */
    movedata(lm_bufsel, 0xC, _my_ds(), (int) devname, strlen(devname) + 1);

    /* Make call */
    bzero(&r, sizeof(r));
    r.x.ax = 0x5F49;
    r.x.ds = lm_bufseg;
    r.x.dx = 0;

    __dpmi_int(0x21, &r);

    /* TODO: Error code */
    if ((r.x.flags & 1) == 1) return(-1);

    /* Copy the data into the buffer */
    movedata(lm_bufsel, 0xC + strlen(devname) + 1, _my_ds(), (int) buf, destbuflen);

    /* Return the level */
    return(0);
}
